// ClientInfo.cpp : Implementation of CClientInfo
#include "stdafx.h"
#include "Dcom_channel_info.h"
#include "ClientInfo.h"
#include "channel_hook_general.h"
#include "tls_manager.h"

/////////////////////////////////////////////////////////////////////////////
// CClientInfo

STDMETHODIMP CClientInfo::GetComputerName(BSTR *computer_name)
{
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    CCallContext* call_info = thread_context->GetLastReceivedCall();

    if (call_info == NULL)
    {
	*computer_name = SysAllocString(L"");
    } else
    {
	USES_CONVERSION;
	WCHAR* w_host_name = A2W(call_info->get_node_info()->computer_name);
	*computer_name = SysAllocString(w_host_name);
    };
    return S_OK;
}

STDMETHODIMP CClientInfo::ImpersonateCaller()
{
    HRESULT hr = S_OK;
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    CCallContext* call_info = thread_context->GetLastReceivedCall();

    if (call_info == NULL)
    {
	return MAKE_HRESULT(1, 0, 0);
    };

    call_info->convert_user_token_to_local_process();

    HANDLE user_token = call_info->get_node_info()->user_token;
    if (user_token != NULL)
    {
	if (user_token == INVALID_HANDLE_VALUE)
	{
	    hr = MAKE_HRESULT(1, 0, 0);	//error
	} else
	{
	    if (!ImpersonateLoggedOnUser(user_token))
	    {
		hr = MAKE_HRESULT(1, 0, 0);	//error
	    };
	};
    } else
    {
	hr = CoImpersonateClient();
    };
    return hr;
}

STDMETHODIMP CClientInfo::RevertToSelf()
{
    HRESULT hr = S_OK;
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    CCallContext* call_info = thread_context->GetLastReceivedCall();
    
    if (call_info != NULL)
    {
	HANDLE user_token = call_info->get_node_info()->user_token;
	if (user_token != NULL)
	{
	    ::RevertToSelf();
	} else
	{
	    hr = CoRevertToSelf();
	};
    } else
    {
	hr = MAKE_HRESULT(1, 0, 0);
    };
    return hr;
}

STDMETHODIMP CClientInfo::GetProcessID(unsigned long *pid)
{
    CThreadContext* thread_context = tls_manager.get_current_thread_context();
    CCallContext* call_info = thread_context->GetLastReceivedCall();

    if (call_info == NULL)
    {
	*pid = 0;
    } else
    {
	*pid = call_info->get_node_info()->process_id;
    };
    return S_OK;
}
