#include "stdafx.h"
#include <assert.h>
#include "channel_hook_general.h"

const IID CHANNEL_HOOK_ID = {0xd9343120, 0x9c1b, 0x11d3, {0xb4, 0x86, 0x0, 0xc0, 0x4f, 0x89, 0x47, 0x7f}};

CCallContext::CCallContext()
{
    node_info.process_id = GetCurrentProcessId();
    gethostname(node_info.computer_name, MAX_NET_COMPUTER_NAME);
    node_info.user_token = NULL;
    b_local_token = false;
}

CCallContext::~CCallContext()
{
    clear();
}

SNodeInfo* CCallContext::get_node_info()
{
    return &node_info;
}

void CCallContext::clear()
{
    node_info.process_id = 0;
    node_info.computer_name[0] = '\0';
    if (node_info.user_token != NULL && b_local_token)
    {
	CloseHandle(node_info.user_token);
	node_info.user_token = NULL;
    };
}

void CCallContext::fill_user_token_info()
{
    node_info.user_token = INVALID_HANDLE_VALUE;
    HANDLE thread_token = NULL;
    DWORD dw;
    bool bresult = OpenThreadToken(GetCurrentThread(), TOKEN_DUPLICATE, true, &thread_token);
    if (!bresult)
    {
	dw = GetLastError();
	if (dw == ERROR_NO_TOKEN)
	{
	    //thread does not have its own security token - try again with process-wide one
	    bresult = OpenProcessToken(GetCurrentProcess(), TOKEN_DUPLICATE, &thread_token);
	    if (!bresult)
	    {
		dw = GetLastError();
	    };
	};
    };
    if (thread_token != NULL)
    {
	bresult = DuplicateTokenEx(thread_token, TOKEN_DUPLICATE | TOKEN_IMPERSONATE | TOKEN_ALL_ACCESS, 
	    NULL, SecurityImpersonation, TokenPrimary, &node_info.user_token);
	if (!bresult)
	{
	    dw = GetLastError();
	    node_info.user_token = INVALID_HANDLE_VALUE;
	} else
	{
	    b_local_token = true;
	};
	CloseHandle(thread_token);
    };
}

bool set_privilege_on_token(
    HANDLE htoken,		    //user token
    const TCHAR* privilege_str,	    //privilege
    bool b_enable		    //enable/disable privilege
    )
{
    TOKEN_PRIVILEGES tp;
    LUID luid;
    TOKEN_PRIVILEGES tp_previous;
    DWORD cb_previous = sizeof(TOKEN_PRIVILEGES);
    bool b_result = false;

    if(LookupPrivilegeValue( NULL, privilege_str, &luid ))
    {
	// first pass.  get current privilege setting
	tp.PrivilegeCount           = 1;
	tp.Privileges[0].Luid       = luid;
	tp.Privileges[0].Attributes = 0;

	if (AdjustTokenPrivileges(htoken, FALSE, &tp, sizeof(TOKEN_PRIVILEGES), &tp_previous, &cb_previous))
	{
	    // second pass.  set privilege based on previous setting
	    tp_previous.PrivilegeCount       = 1;
	    tp_previous.Privileges[0].Luid   = luid;

	    if(b_enable) 
	    {
		tp_previous.Privileges[0].Attributes |= (SE_PRIVILEGE_ENABLED);
	    } else 
	    {
		tp_previous.Privileges[0].Attributes ^= (SE_PRIVILEGE_ENABLED &
		    tp_previous.Privileges[0].Attributes);
	    }
	    if (AdjustTokenPrivileges(htoken, FALSE, &tp_previous, cb_previous, NULL, NULL))
	    {
		b_result = true;
	    };
	};
    };

    DWORD dw = GetLastError();

    return b_result;
}

void CCallContext::convert_user_token_to_local_process()
{
    if (node_info.user_token != NULL && node_info.user_token != INVALID_HANDLE_VALUE && !b_local_token)
    {
	//check if we do it on the local machine
	//delegation is not supported now on cross-machine basises
	TCHAR buffer[MAX_NET_COMPUTER_NAME];
	gethostname(buffer, MAX_NET_COMPUTER_NAME);
	if (_tcscmp(buffer, node_info.computer_name) == 0)
	{
	    //get handle of the calling process

	    HANDLE calling_process_handle = NULL;
	    HANDLE htoken = NULL;
	    //enable DEBUG privilege for PF Server - otherwise it will be unable to OpenProcess
	    BOOL b = OpenProcessToken(GetCurrentProcess(), TOKEN_ADJUST_PRIVILEGES | TOKEN_QUERY, &htoken);
	    if (htoken != NULL)
	    {
		b = set_privilege_on_token(htoken, SE_DEBUG_NAME, TRUE);
		b = set_privilege_on_token(htoken, SE_SECURITY_NAME , TRUE);
		b = set_privilege_on_token(htoken, SE_AUDIT_NAME, TRUE);

		calling_process_handle = OpenProcess(PROCESS_DUP_HANDLE | PROCESS_ALL_ACCESS, FALSE, node_info.process_id);

		if (calling_process_handle != NULL)
		{
		    HANDLE new_handle;
		    BOOL bresult = DuplicateHandle(calling_process_handle, node_info.user_token, 
			GetCurrentProcess(), &new_handle, 0, FALSE, DUPLICATE_SAME_ACCESS );
		    if (bresult)
		    {
			node_info.user_token = new_handle;
			b_local_token = true;
		    } else
		    {
			DWORD dw = GetLastError();
			node_info.user_token = INVALID_HANDLE_VALUE;
		    };
		    CloseHandle(calling_process_handle);
		} else
		{
		    node_info.user_token = INVALID_HANDLE_VALUE;
		    DWORD dw = GetLastError();
		};

		CloseHandle(htoken);
	    } else
	    {
		assert(false);
		node_info.user_token = INVALID_HANDLE_VALUE;
	    };

	} else
	{
	    node_info.user_token = NULL;
	};
    };
}

CThreadContext::~CThreadContext()
{
    while (sent_client_info.size() != 0)
    {
	delete sent_client_info.top();
	sent_client_info.pop();
    };

    while (recieved_client_info.size() != 0)
    {
	delete recieved_client_info.top();
	recieved_client_info.pop();
    };
}

CCallContext* CThreadContext::GetLastReceivedCall()
{
    CCallContext* res_call = NULL;
    if (recieved_client_info.size() != 0)
    {
	res_call = recieved_client_info.top();
    };
    return res_call;
}
